/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HTTP_Server.c
 * Purpose: HTTP Server example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include "RTE_Components.h" 
#include "cmsis_os2.h"

#include "rl_net.h"                     // Keil.MDK-Pro::Network:CORE
#include "rsi_board.h"
#include "rsi_chip.h"

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#define USER_BUTTON    1
uint64_t app_main_stk[];

const osThreadAttr_t app_main_attr;

void app_main (void *arg);

// Main stack size must be multiple of 8 Bytes
#define APP_MAIN_STK_SZ (1024U)
uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

extern uint8_t  get_button     (void);
extern void     netDHCP_Notify (uint32_t if_num, uint8_t option, const uint8_t *val, uint32_t len);


bool LEDrun;

/* Thread IDs */

osThreadId_t TID_Led;

/* Thread declarations */
static void BlinkLed (void *arg);


/**
 * @brief  Get the button status on evalutation board
 * @param  None
 * @retval return the button status
 */
uint32_t GetButtonState (void)
{
uint32_t val=0;
	
	if(RSI_EGPIO_GetPin(EGPIO1,0,12) == ENABLE){
     val |=USER_BUTTON;
  }
	if(RSI_NPSSGPIO_GetPin(NPSS_GPIO_2) == ENABLE){
		val |=(USER_BUTTON << 1);
	}
	return val;
}

/**
 * @brief  Initialize buttons
 * @param  None
 * @retval None
 */
void Buttons_Initialize(void)
{
	/*Enable clcok for gpio*/
  RSI_ULPSS_PeripheralEnable(ULPCLK,ULP_EGPIO_CLK,ENABLE_STATIC_CLK);
	
	/*Initialize the user switch*/
	RSI_EGPIO_UlpPadReceiverEnable(12);
	RSI_EGPIO_SetPinMux(EGPIO1,0,12,EGPIO_PIN_MUX_MODE0);
	RSI_EGPIO_SetDir(EGPIO1,0,12,EGPIO_CONFIG_DIR_INPUT);
	/*Initialize the wakeup switch*/
	RSI_NPSSGPIO_InputBufferEn(NPSS_GPIO_2,1);
	RSI_NPSSGPIO_SetPinMux(NPSS_GPIO_2,NPSSGPIO_PIN_MUX_MODE0);
	RSI_NPSSGPIO_SetDir(NPSS_GPIO_2,NPSS_GPIO_DIR_INPUT);
}

/* Read digital inputs */
uint8_t get_button (void) {
  return ((uint8_t)GetButtonState ());
}


/* IP address change notification */
void netDHCP_Notify (uint32_t if_num, uint8_t option, const uint8_t *val, uint32_t len) {
   char ip_ascii[16];
  
  if (option == NET_DHCP_OPTION_IP_ADDRESS) {
        netIP_ntoa(NET_ADDR_IP4,&val[0],ip_ascii,sizeof(ip_ascii));
   // 		DEBUGOUT("IP4 address: %s",ip_ascii);
	}
}

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static __NO_RETURN void BlinkLed (void *arg) {
  const uint8_t led_val[] = { 0, 1, 2 };
  int cnt = 0;

  LEDrun = true;
  while(1) {
    // Every 100 ms
    if (LEDrun == true) {
      RSI_Board_LED_Toggle (led_val[cnt]);
      if (++cnt >= sizeof(led_val)) {
        cnt = 0;
      }
    }
    osDelay (100);
  }
}


/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *arg) {
  (void)arg;

#ifdef RTE_Compiler_EventRecorder
  EventRecorderInitialize(EventRecordAll, 1U);
#endif
  RSI_Board_Init();

  Buttons_Initialize();

	netInitialize ();

  TID_Led     = osThreadNew (BlinkLed, NULL, NULL);

  while(1) {

    osThreadFlagsWait (0, osFlagsWaitAny, osWaitForever);
  }
}
