/*----------------------------------------------------------------------------

 *---------------------------------------------------------------------------*/

#include "cmsis_os2.h"
#include "rsi_board.h"

#include "RTE_Components.h"

static osThreadId_t tid_phaseA;                  /* Thread id of thread: phase_a      */
static osThreadId_t tid_phaseB;                  /* Thread id of thread: phase_b      */

#define LED_n   0

/*----------------------------------------------------------------------------
 *      Switch LED on
 *---------------------------------------------------------------------------*/
void Switch_On (unsigned char led) {
  RSI_Board_LED_Set(led,0);
}

/*----------------------------------------------------------------------------
 *      Switch LED off
 *---------------------------------------------------------------------------*/
void Switch_Off (unsigned char led) {
  RSI_Board_LED_Set(led,1);
}


/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId_t tid)  {
  osDelay(500);                             /* delay 500ms                   */
  osThreadFlagsSet (tid, 0x0001);                 /* set signal to thread 'thread' */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_On (LED_n);
    signal_func(tid_phaseB);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_Off(LED_n);
    signal_func(tid_phaseA);                /* call common signal function   */
  }
}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *argument) {

  (void)argument;

  tid_phaseA = osThreadNew (phaseA, NULL, NULL);            /* create phaseA thread */
  if (tid_phaseA == NULL) { /* add error handling */ }

  tid_phaseB = osThreadNew (phaseB, NULL, NULL);            /* create phaseB thread */
  if (tid_phaseB == NULL) { /* add error handling */ }
	
	osThreadFlagsSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */
	
  for (;;) {}
}


/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  RSI_Board_Init();                         /* Initialize the LEDs           */
	
  /* Initialize CMSIS-RTOS2 */
  osKernelInitialize ();

  /* Create application main thread */
  osThreadNew(app_main, NULL, NULL);

  /* Start thread execution */
  osKernelStart();

  while(1);
}
