/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HID.c
 * Purpose: USB Device - Human Interface Device example
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_usb.h"                     /* RL-USB function prototypes         */
#include "rsi_chip.h"
#include "rsi_board.h"

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#define USER_BUTTON    1
extern uint8_t  get_button     (void);
/**
 * @brief  Get the button status on evalutation board
 * @param  None
 * @retval return the button status
 */
uint32_t GetButtonState (void)
{
  uint32_t val=0;
	
	if(RSI_EGPIO_GetPin(EGPIO1,0,12) == ENABLE){
     val |=USER_BUTTON;
  }
	if(RSI_NPSSGPIO_GetPin(NPSS_GPIO_2) == ENABLE){
		val |=(USER_BUTTON << 1);
	}
	return val;
}


/**
 * @brief  Initialize buttons
 * @param  None
 * @retval None
 */
void Buttons_Initialize(void)
{
	/*Enable clcok for gpio*/
  RSI_ULPSS_PeripheralEnable(ULPCLK,ULP_EGPIO_CLK,ENABLE_STATIC_CLK);
	
	/*Initialize the user switch*/
	RSI_EGPIO_UlpPadReceiverEnable(12);
	RSI_EGPIO_SetPinMux(EGPIO1,0,12,EGPIO_PIN_MUX_MODE0);
	RSI_EGPIO_SetDir(EGPIO1,0,12,EGPIO_CONFIG_DIR_INPUT);
	/*Initialize the wakeup switch*/
	RSI_NPSSGPIO_InputBufferEn(NPSS_GPIO_2,1);
	RSI_NPSSGPIO_SetPinMux(NPSS_GPIO_2,NPSSGPIO_PIN_MUX_MODE0);
	RSI_NPSSGPIO_SetDir(NPSS_GPIO_2,NPSS_GPIO_DIR_INPUT);
}

/* Read digital inputs */
uint8_t get_button (void) {
  return ((uint8_t)GetButtonState ());
}


/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
int main (void) {
  uint8_t but;
  uint8_t buf[2] = { 0, 0 };
		
	RSI_CLK_M4SocClkConfig(M4CLK,M4_ULPREFCLK,0);
	/*configure the system clcok to 180MHZ*/
	RSI_CLK_SetSocPllFreq(M4CLK,180000000,40000000);
	
	RSI_CLK_M4SocClkConfig(M4CLK,M4_SOCPLLCLK,0);
	#ifdef RTE_Compiler_EventRecorder
  EventRecorderInitialize(EventRecordAll, 1U);
	#endif
	/*Board initialization for leds*/
	RSI_Board_Init();
  /*Button initialization*/  
	Buttons_Initialize();
	
	USBD_Initialize         (0);          /* USB Device Initialization        */
  USBD_Connect            (0);          /* USB Device Connect               */
  
  while (1) {                           /* Loop forever                       */
    but = (uint8_t)(get_button ());
    if ((but ^ buf[0]) & 3) {
      buf[0] = but & 3;
      USBD_HID_GetReportTrigger(0, 0, &buf[0], 1);
    }
    if (((but >> 2) ^ buf[1]) & 3) {
      buf[1] = (but >> 2) & 3;
      USBD_HID_GetReportTrigger(1, 0, &buf[1], 1);
    }
    osDelay(100);                       /* 100 ms delay for sampling buttons  */
  }
}
