/**
 * @file       rsi_dac_static_mode_example.c
 * @version    1.0
 * @date       05 Feb 2019
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This file contains example to describe DAC functionality.
 *
 * @section Description
 * Demonstrate the DAC functionality in static mode. 
 */
 
/* Includes ------------------------------------------------------------------*/
#include "rsi_chip.h"

/* Private typedef -----------------------------------------------------------*/

/* Private macro -------------------------------------------------------------*/
#define  OUTPUT_PIN_NUMBER        4
#define  OUTPUT_MUX_ENABLE        1
#define  OUTPUT_MUX_SEL           0   /* Select DAC output on ULP_GPIO4 */
#define  DAC_STATIC_MODE_ENABLE   1
#define  CLK_DIV                  4   /* Configure DAC sampling frequency as 5Msps (40Mhz/2*4)*/
#define  PRBS_SEL                 0   /* Switch off PRBS RRBS for DAC */
#define  SAMPLE_LENGTH            1

/* Private Veriable ---------------------------------------------------------*/ 
volatile uint8_t ping_pong_cntr=0,intr_done;
/* Standard sin wave samples */
uint16_t input_sample_data[SAMPLE_LENGTH] = {0x33A};

/**
 * @brief      This API configure the pin for DAC output.
 * @param[in]  Pin number to use play DAC output on GPIO , (ULP_GPIO4 or ULP_GPIO15)
 * @return     None
 */ 
void dac_output_pin_mux(uint16_t pin_number)
{
	RSI_EGPIO_SetPinMux(EGPIO1,0,pin_number,EGPIO_PIN_MUX_MODE7);
}

/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */ 
int main(void)
{
		/* At this stage the MICROCONTROLLER clock setting is already configured,
		* this is done through SystemInit() function which is called from startup
		* file (startup_rs1xxxx.s) before to branch to application main.
		* To reconfigure the default setting of SystemInit() function, refer to
		* system_rs1xxxx.c file
		*/
		SystemCoreClockUpdate();	
	
	  /* Set ULP reference clock as RF ref clock (default rf ref clock is 40Mhz XTAL)*/	
		RSI_ULPSS_RefClkConfig(ULPSS_RF_REF_CLK);		
	
		/* Power gate enable for DAC */
		RSI_DAC_PowerControl(DAC_POWER_ON);
	
		/* 40 MHZ XTAL Clock configuration of DAC */	
  	RSI_ULPSS_AuxClkConfig(ULPCLK, ENABLE_STATIC_CLK ,ULP_AUX_REF_CLK);
	
		/* Configure the ULP_GPIO4 as analog mode for DAC output */
		dac_output_pin_mux(OUTPUT_PIN_NUMBER);	
	
		/* Configure the reference LDO voltage as 2.8v */		
	  RSI_ADC_AUXLdoConfig(AUX_ADC_DAC_COMP,0,0xf);
	
		/* Divide DAC clock */ 
		RSI_DAC_ClkDivFactor(AUX_ADC_DAC_COMP, CLK_DIV);
	
		/* Configure the DAC in FIFO mode */
		RSI_DAC_Config(AUX_ADC_DAC_COMP,DAC_STATIC_MODE_ENABLE,OUTPUT_MUX_ENABLE,OUTPUT_MUX_SEL,PRBS_SEL);
	
	  RSI_DAC_Start(AUX_ADC_DAC_COMP,ENABLE);
		
		/* Write data in DAC Input register */
		RSI_DAC_WriteData(AUX_ADC_DAC_COMP,input_sample_data,DAC_STATIC_MODE_ENABLE,SAMPLE_LENGTH);
  
	  while(1);
}

