 /**
 * @file       uulp_pin_interrupt.c
 * @version    1.1.0
 * @date       13th Mar 2019
 *
 * Copyright(C) Redpine Signals 2019
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This files contains example code of all end to end functionality
 *
 * @section Description
 * This example test the all GPIOs(M4SS,ULPSS,UULP) toggle status.
 *
 *
 */
/* Includes -------------------------------------------------------------------------------------------------------*/
#include "rsi_chip.h"
#include "rsi_board.h"

/* Private typedef ------------------------------------------------------------------------------------------------*/

/* Private define -------------------------------------------------------------------------------------------------*/
#define M4_GPIO_PIN
//#define ULP_GPIO_PIN
//#define UULP_GPIO_PIN
/* Private macro --------------------------------------------------------------------------------------------------*/
#define PORT 0
#define PIN  6
/* Private variables ----------------------------------------------------------------------------------------------*/

/* Private function prototypes ------------------------------------------------------------------------------------*/

/* Private functions ----------------------------------------------------------------------------------------------*/

/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */
int main(void)
{
	/* At this stage the microcontroller clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs9116.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs9116.c file
	 */
	int forever = 1,pad_sel;

	/*Configures the system default clock and power configurations*/
	SystemCoreClockUpdate();

#ifdef M4_GPIO_PIN
	if(PIN >24 && PIN <31)
	{
		RSI_EGPIO_HostPadsGpioModeEnable(PIN);
	}
	/*Enable pad selection for gpio pins*/
	for(pad_sel =0 ;pad_sel <22;pad_sel++)
	{
		RSI_EGPIO_PadSelectionEnable(pad_sel);
	}
	/* Set pin 0 in GPIO mode */
	RSI_EGPIO_SetPinMux(EGPIO ,PORT , PIN ,EGPIO_PIN_MUX_MODE0);

	/* Set output direction */
	RSI_EGPIO_SetDir(EGPIO ,PORT ,PIN, EGPIO_CONFIG_DIR_OUTPUT);

	while (forever)
	{
		/* Toggle gpio in loop*/
		RSI_EGPIO_SetPin(EGPIO ,PORT,PIN ,0);
		RSI_EGPIO_SetPin(EGPIO ,PORT,PIN ,1);
	}
#endif

#ifdef ULP_GPIO_PIN
	/* Set pin 0 in GPIO mode */
	RSI_EGPIO_SetPinMux(EGPIO1 ,PORT , PIN ,EGPIO_PIN_MUX_MODE0);
	/* Set output direction */
	RSI_EGPIO_SetDir(EGPIO1 ,PORT ,PIN, EGPIO_CONFIG_DIR_OUTPUT);
	while(1)
	{
		/* Toggle gpio in loop*/
		RSI_EGPIO_SetPin(EGPIO1 ,PORT,PIN ,1);
		RSI_EGPIO_SetPin(EGPIO1 ,PORT,PIN ,0);
	}
#endif

#ifdef UULP_GPIO_PIN
	/*NPSS GPIO */
	RSI_NPSSGPIO_InputBufferEn(PIN , 1U);

	RSI_NPSSGPIO_SetPinMux(PIN , 0);

	RSI_NPSSGPIO_SetDir(PIN , NPSS_GPIO_DIR_OUTPUT);

	while(1)
	{
		/* Toggle gpio in loop*/
		RSI_NPSSGPIO_SetPin(PIN,0U);
		RSI_NPSSGPIO_SetPin(PIN,1U);
	}
#endif
	
  /*Statement will never reach here , just to satisfy the standard main*/
	return 0;
}
