 /**
 * @file       Sdio_Host_Exp.c
 * @version    1.0
 * @date       10th AUG 2018
 *
 * Copyright(C) Redpine Signals 2018
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This files contains example code of all end to end functionality
 *
 * @section Description
 * This file contains example code to demonstrate the SDIO Host functionality.
 *
 *
 */

/* Includes -------------------------------------------------------------------------------------------------------*/
#include "rsi_chip.h"
#include "rsi_board.h"
/* Private typedef ------------------------------------------------------------------------------------------------*/

/* Private define -------------------------------------------------------------------------------------------------*/

/* Private macro --------------------------------------------------------------------------------------------------*/
#define BLOCK_SIZE                1024
#define SOC_OPER_FREQUENCY        30000000 /*220Mhz*/
#define SOC_OPER_FREQUENCY_SIM    200000000 /*220Mhz*/
#define MEMORY_LOCATION           0x16000        //16 bit address and 17th bit 1 for memory data writes


/* Private variables ----------------------------------------------------------------------------------------------*/
uint8_t Write_Data[BLOCK_SIZE];
uint8_t Read_Data[BLOCK_SIZE]={0};
volatile uint8_t ret =1;


/* Private functions ----------------------------------------------------------------------------------------------*/
void smih_callback_handler(uint32_t event)
  {  
    switch(event)
    {
     case COMMAND_COMPLETE   :
       break;
     case TRANSFER_COMPLETE  :
       break;
      case CARD_INSERTION    :
       break;    
      case CARD_REMOVAL      :
        break;
      case CARD_INTERRUPT    :        
        break;
    }  
  }

/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */ 
int main(void)
{
	/* At this stage the microcontroller clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs9116.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs9116.c file
	 */

	SMIH_CARD_CONFIG_T SdiohConfg ;
	uint32_t i = 0;
    
  /* Clear structure */
  memset(&SdiohConfg, 0, sizeof(SdiohConfg));
  
	SdiohConfg.highSpeedEnable    = 0;
	SdiohConfg.uhsModeSelect      = UHS_NONE;      //uhs mode will work only in 4bit mode
	SdiohConfg.admaMode           = 0;
	SdiohConfg.busWidthMode       = SMIH_BUS_WIDTH4;
	SdiohConfg.clock              = 25000000;
	SdiohConfg.voltage            = 0; //set this bit in case of 1.8v
	SdiohConfg.funcNum            = 1;
	SdiohConfg.blockModeEnable    = 1; //enable in case of block mode
	SdiohConfg.opCode             = 1;
	SdiohConfg.byteBlockSize      = 256;
	SdiohConfg.numberOfBlocks     = 4;

	/*Configures the system default clock and power configurations*/
	SystemCoreClockUpdate();

  /*Systic Config*/
	SysTick_Config(SOC_OPER_FREQUENCY_SIM / 1000);

	/*Configure Source clock and  module clock for SDMEM */
	RSI_CLK_SetSocPllFreq(M4CLK,50000000,40000000);
	RSI_CLK_M4SocClkConfig(M4CLK,M4_SOCPLLCLK,0);
	RSI_CLK_SdMemClkConfig(M4CLK ,1,SDMEM_SOCPLLCLK,1);

	/*Smih IO Card Initialization*/
	if(RSI_OK == RSI_SDIOH_Enumeration(&SdiohConfg,smih_callback_handler))
	{
		/* init success */
		for(i = 0; i<BLOCK_SIZE;i++)
		{      
			Write_Data[i] =i+1;			
		}
	}
	else
	{
		while(1); // Initialization failed
	}

  /* Increase clock to 25MHZ after enumeration*/
	smih_clock_config(&SdiohConfg,25000000);

	/*Write Data to memory*/
	if(RSI_OK != RSI_SDIOH_ByteBlockWriteCmd53 (&SdiohConfg ,Write_Data,MEMORY_LOCATION))
	{
		/* write fail */
		return ERROR_SMIH;
	}

	/*read data from memory*/
	if(RSI_OK != RSI_SDIOH_ByteBlockReadCmd53(&SdiohConfg ,Read_Data,MEMORY_LOCATION))
	{
		/* read fail */
		return ERROR_SMIH;
	}
  	
  /* compare data */
  ret = memcmp(Write_Data, Read_Data,sizeof(Write_Data));

  while(1);
}
