 /**
 * @file       rsi_sdioh.h
 * @version    1.0
 * @date       10th AUG 2018
 *
 * Copyright(C) Redpine Signals 2018
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This files contains functions prototypes related to SMIH Master
 *
 * @section Description
 * This file contains the list of function prototypes for the SMIH Master and low level function definitions
 * Following are list of API's which need to be defined in this file.
 *
 */

/**
 * Includes
 */
#ifndef  __RSI_SDMEM_H__
#define  __RSI_SDMEM_H__

#ifdef __cplusplus
extern "C"
{
#endif

#include "RS1xxxx.h"
#include "base_types.h"
#include "rsi_smih.h"  


#define SET1    0
#define SET2    1

#if SET1
#define SD_CLK_PIN      46
#define SD_CLK_PAD      11
#define SD_CLK_MUX      EGPIO_PIN_MUX_MODE8

#define SD_CMD_PIN      47
#define SD_CMD_PAD      11
#define SD_CMD_MUX      EGPIO_PIN_MUX_MODE8

#define SD_D0_PIN       48
#define SD_D0_PAD       11
#define SD_D0_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D1_PIN       49
#define SD_D1_PAD       12
#define SD_D1_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D2_PIN       50
#define SD_D2_PAD       12
#define SD_D2_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D3_PIN       51
#define SD_D3_PAD       12
#define SD_D3_MUX       EGPIO_PIN_MUX_MODE8

#define SD_CD_PIN       53
#define SD_CD_PAD       13
#define SD_CD_MUX       EGPIO_PIN_MUX_MODE8

#define SD_WP_PIN       52
#define SD_WP_PAD       13
#define SD_WP_MUX       EGPIO_PIN_MUX_MODE8
#endif

#if SET2
#define SD_CLK_PIN      25
#define SD_CLK_PAD      0 //no pad
#define SD_CLK_MUX      EGPIO_PIN_MUX_MODE8

#define SD_CMD_PIN      26
#define SD_CMD_PAD      0 //no pad
#define SD_CMD_MUX      EGPIO_PIN_MUX_MODE8

#define SD_D0_PIN       27
#define SD_D0_PAD       0 //no pad
#define SD_D0_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D1_PIN       28
#define SD_D1_PAD       0 //no pad
#define SD_D1_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D2_PIN       29
#define SD_D2_PAD       0 //no pad
#define SD_D2_MUX       EGPIO_PIN_MUX_MODE8

#define SD_D3_PIN       30
#define SD_D3_PAD       0 //no pad
#define SD_D3_MUX       EGPIO_PIN_MUX_MODE8

#define SD_CD_PIN       53
#define SD_CD_PAD       13
#define SD_CD_MUX       EGPIO_PIN_MUX_MODE8

#define SD_WP_PIN       52
#define SD_WP_PAD       13
#define SD_WP_MUX       EGPIO_PIN_MUX_MODE8
#endif


#define ACMD41_VOLTAGE               (0x00FF8000)
#define ACMD41_UHS_REQ               (0x41FF8000)
#define ACMD41_HCS                   (1<<30)

#define SD_ACMD_OFFSET               (0x40)
#define MMC_CMD_TAG                  (0x80)
#define ACMD41_BUSY_BIT              BIT(31)
#define ACMD41_OCR_BIT               BIT(30)
                    
/****************SDMEM related function prototypes**************************/
void RSI_SDMEM_PinMux(void);
error_t RSI_SDMEM_GoIdleStateCmd0( SMIH_CARD_CONFIG_T *pSmihConfig );
error_t RSI_SDMEM_SendCardInterfaceConditionCmd8(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMEM_SendApplicationCommandCmd55(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMEM_SetCardOperationConditionAcmd41(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMEM_SwitchVoltageCmd11(SMIH_CARD_CONFIG_T *pSmihConfig, uint32_t CmdArg);
error_t RSI_SDMEM_SendCidCmd2(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDIOH_SendRelativeAddressCmd3(SMIH_CARD_CONFIG_T *pSmihConfig, uint32_t CmdArg);
error_t RSI_SDMEM_SendCsdCmd9(SMIH_CARD_CONFIG_T *pSmihConfig);
error_t RSI_SDMEM_SelectCardCmd7(SMIH_CARD_CONFIG_T *pSmihConfig);
error_t RSI_SDMEM_SetBusWidthAcmd6(SMIH_CARD_CONFIG_T *pSmihConfig, uint32_t Arg);
error_t RSI_SDMEM_OperationSwitchFunctionReadCmd6(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Argument);
error_t RSI_SDMEM_GetCardOperationConditionAcmd41(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMEM_CardBusWidthConfig(SMIH_CARD_CONFIG_T *pSmihConfig, uint8_t BusWidth);
error_t RSI_SDMEM_OperationSwitchFunctionWriteCmd6(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Argument);
error_t RSI_SDMEM_BlocksWrite(SMIH_CARD_CONFIG_T *pSmihConfig,const uint8_t *DataIn,uint32_t BlockIndex,uint32_t BlockCount);
error_t RSI_SDMEM_BlocksRead(SMIH_CARD_CONFIG_T *pSmihConfig,uint8_t *DataOut,uint32_t BlockIndex,uint32_t BlockCount);
error_t RSI_SDMEM_EnableHighSpeed(SMIH_CARD_CONFIG_T *pSmihConfig,boolean_t HighSpeed, uint32_t Clock);
error_t RSI_SDMEM_Enumeration(SMIH_CARD_CONFIG_T *pSmihConfig,ARM_SMIH_SignalEvent_t Event);
error_t RSI_SDMMC_Enumeration(SMIH_CARD_CONFIG_T *pSmihConfig,ARM_SMIH_SignalEvent_t Event);
error_t RSI_SDMMC_SendOperationConditionCmd1(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMMC_SendExtentCsdCmd(SMIH_CARD_CONFIG_T *pSmihConfig);
error_t RSI_SDMMC_ChangeBusWidthMode(SMIH_CARD_CONFIG_T *pSmihConfig,uint8_t bus_wdith);
error_t RSI_SDMMC_SwitchFunctionCmd6(SMIH_CARD_CONFIG_T *pSmihConfig,uint32_t Arg);
error_t RSI_SDMMC_HighSpeedMode(SMIH_CARD_CONFIG_T *pSmihConfig,boolean_t HighSpeed, uint32_t Clock);

#ifdef __cplusplus
}
#endif

#endif  /*__RSI_SDMEM_H__*/

/* @}end of  RSI_EGPIO_DRIVERS */
