/**
 * @file       rsi_egpio.h
 * @version    0.9.3
 * @date       6th March,2017
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This files contains functions prototypes related to EGPIO peripheral
 * 
 * @section Description
 * This file contains the list of function prototypes for the EGPIO and low level function definitions
 * Following are list of API's which need to be defined in this file.
 *
 */


/**
 * Includes
 */
#ifndef __RSI_EGPIO_H__
#define __RSI_EGPIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "stdint.h"
#include "string.h"

#define  BIT(x)    ((uint32_t)1 << (x))

/*****m4 PAD configuration defines*****/

#define PAD_CONFIG_REG(x)     (* (volatile uint32_t *) (0x46004000 +4*x))  			/*!< REN enable bit(this should be enable)                              */
#define PADSELECTION          (* (volatile uint32_t *) (0x41300000+0x610))  		/*!< PAD selection (0 t0 21) A value of 1 on this gives control to M4SS */
#define EGPIO_MUX(p,n)        (* (volatile uint32_t *) (0x46130000 + ((p*16)+n)*16)) /*!<SetPinMux>*/

void RSI_EGPIO_SetPinMux(uint8_t port , uint8_t pin , uint8_t u8Mux);

void RSI_EGPIO_PadSelectionEnable(uint8_t padNum);

void RSI_EGPIO_PadSelectionDisable(uint8_t padNum);

void RSI_EGPIO_PadReceiverEnable(uint8_t u8GpioNum);

#ifdef __cplusplus
}
#endif

#endif /* __RSI_EGPIO_H__*/
